#include "/AGONY-U2/paciorek/netcdf/netcdf-3.4/src/cxx/netcdf.hh"
#include <fstream.h>
#include <iostream.h>
#include <iomanip.h>
#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#define PI 3.141593
#define LON 144
#define LAT 37
#define TIME 18900 // 1/1/48-9/29/99
#define R 287 // J/(K-kg) - right - it shouldn't be universal gas constant 8314 J/(K-kmol)
#define Cp 1004 // J/(kg-K)
#define g 9.81 // m/s-2  - right? it shouldn't be grav.constant 6.673x10-11 N m^2 / (kg^2) - also, sea level g is ok?
#define P 50000 // 500 hPa - should this be expressed as kPa??
#define scale_factor 0.01
#define add_offset 477.66
#define scale_factor_N2 10000000
#define add_offset_N2 0


void compute_N2(short *orig_temp500,short *orig_temp400,short *orig_temp600,short *N2,int lon,int lat)
{
  int day, locn; long tmp2; double sigma, temp, tmpval, tmpval2;
  for(day=0;day<TIME;day++)
    {
      locn=day*LON+lon;
      temp=(double) scale_factor*orig_temp500[locn]+add_offset;
      tmpval=(double) g*g*( (-P*scale_factor*(orig_temp400[locn]-orig_temp600[locn]))/(R*temp*temp*(100*(400-600))) + (1/(Cp*temp)));
      tmpval*=scale_factor_N2;
      N2[day]=(short) tmpval;
      tmpval2=tmpval*10;
      tmp2=(long) tmpval2;
      if((tmp2%10)>=5) 
	{
	  if(tmpval>=0) {N2[day]++;}
	  else {N2[day]--;}
	}
    }
}


int main()
{
  NcBool recd;
	
  

  long c0,c1,c2,c3;
  long cur[4];
  long cur_out[3];
  
  const char* path1="/tmp/paciorek/temp.500.nc";
  NcFile file(path1,NcFile::ReadOnly);
  if (! file.is_valid()) {
    cerr << "can't read netCDF file " << path1 << endl;
    return 0;
  }

  const char* path2="/tmp/paciorek/temp.400.nc";
  NcFile file2(path2,NcFile::ReadOnly);
  if (! file2.is_valid()) {
    cerr << "can't read netCDF file " << path2 << endl;
    return 0;
  }

  const char* path3="/tmp/paciorek/temp.600.nc";
  NcFile file3(path3,NcFile::ReadOnly);
  if (! file3.is_valid()) {
    cerr << "can't read netCDF file " << path3 << endl;
    return 0;
  }


  const char* path4="/tmp/paciorek/N2.500.nc";
  NcFile output(path4,NcFile::New); // change to New once I get this working
  if (! output.is_valid()) {
    cerr << "can't create netCDF file " << path4 << endl;
    return 0;
  }
  
  //input
  NcVar *temp500_in, *temp400_in, *temp600_in, *lat_in,*lon_in, *time_in;

  //output
  NcDim *timedim,*latdim, *londim;
  NcVar *time_out, *N2_out, *lat_out, *lon_out;
  NcType type;

  //initializing output file:

  cout << "Creating: " << path4 << endl;

  
  

  londim=output.add_dim("lon",LON);
  latdim=output.add_dim("lat",LAT);
  timedim=output.add_dim("time",TIME);

  // Doing monthly tempvar output

  type=(NcType) 5; // float
  lon_out=output.add_var("lon",type,londim);
  type=(NcType) 5;  // float
  lat_out=output.add_var("lat",type,latdim);
  type=(NcType) 3; // short  
  N2_out=output.add_var("N2",type,timedim,latdim,londim);
  type=(NcType) 6;  // double since it's daily
  time_out=output.add_var("time",type,timedim);

  // need to check that lat and lon are indeed floats


  recd=output.add_att("title","static stability\0");
  recd=output.add_att("history","created by C. Paciorek, Department of Statistics, Carnegie Mellon University, paciorek@stat.cmu.edu on 4/23/00\0");
  recd=output.add_att("description","N^2, the static stability, follows Lee and Mak (1994) J Atmos Sci 51:2137 and is calculated at 500 mb for all days from 1/1/48-9/29/99 at all Northern hemisphere latitudes and longitudes\0"); 

  nclong range[2];
  range[0]=17067072;
    range[1]=17520648;
  recd=(*time_out).add_att("units","hours since 1-1-1 00:00:0.0\0");
  recd=(*time_out).add_att("long_name","Time\0");
  recd=(*time_out).add_att("actual_range",2,range);
  recd=(*time_out).add_att("delta_t","0000-00-01 00:00:00\0");
  recd=(*time_out).add_att("avg_period","0000-00-01 00:00:00\0");

  double factor=(double) 1/scale_factor_N2;

  recd=(*N2_out).add_att("long_name","static stability\0");
  recd=(*N2_out).add_att("var_desc","static stability, N^2, at 500 mb\0");
  recd=(*N2_out).add_att("units","sec^-2\0");  // check this
  recd=(*N2_out).add_att("add_offset",add_offset_N2);
  recd=(*N2_out).add_att("scale_factor",factor);

  recd=(*lat_out).add_att("long_name","Latitude\0");
  recd=(*lat_out).add_att("units","degrees_east");

  float range2[2];
  range2[0]=0; range2[1]=357.5;
  recd=(*lat_out).add_att("actual_range",2,range2);
  
  recd=(*lon_out).add_att("long_name","Longitude\0");
  recd=(*lon_out).add_att("units","degrees_north");
  range2[0]=90.0; range2[1]=0.0;
  recd=(*lon_out).add_att("actual_range",2,range2);
  
  //reading and manipulating data:

  temp500_in=file.get_var("air"); 
  temp400_in=file2.get_var("air"); 
  temp600_in=file3.get_var("air"); 
  

   short orig_temp500[(TIME*LON)];
   short orig_temp400[(TIME*LON)];
   short orig_temp600[(TIME*LON)];

  short N2[TIME];

  
  int lat, lon, k, time;


  cout << "beginning lat/lon cycle...";
  cout.flush();

  //for(lat=7;lat<8;lat++)
  for(lat=0;lat<LAT;lat++)
    {
      cout << endl;
      cout << lat << " ";
      cout.flush();
      //for(lon=5;lon<6;lon++)
      
      cur[0]= 0; cur[1]=0; cur[2]= lat; cur[3]= 0; 
      (*temp500_in).set_cur(cur);
      c0=TIME; c1=1; c2=1; c3=LON;
      recd=(*temp500_in).get(orig_temp500,c0,c1,c2,c3);
      
      cur[0]= 0; cur[1]=0; cur[2]= lat; cur[3]= 0; 
      (*temp400_in).set_cur(cur);
      c0=TIME; c1=1; c2=1; c3=LON;
      recd=(*temp400_in).get(orig_temp400,c0,c1,c2,c3);
      
      cur[0]= 0; cur[1]=0; cur[2]= lat; cur[3]= 0; 
      (*temp600_in).set_cur(cur);
      c0=TIME; c1=1; c2=1; c3=LON;
      recd=(*temp600_in).get(orig_temp600,c0,c1,c2,c3);
      
      for(lon=0;lon<LON;lon++)
	{
	  cout << lon << " ";
	  cout.flush();

	  compute_N2(orig_temp500,orig_temp400,orig_temp600,N2,lon,lat);


	  cur_out[0]=0;
	  cur_out[1]=lat;
	  cur_out[2]=lon;
	  (*N2_out).set_cur(cur_out);
	  c0=TIME; c1=1; c2=1;
	  recd=(*N2_out).put(N2,c0,c1,c2);
	  

	}
    }			
  
  
  
  lat_in=file.get_var("lat");
  c1=LAT;
  float lats[LAT];
  recd=(*lat_in).get(lats,c1);
  recd=(*lat_out).put(lats,c1);

  
  
  lon_in=file.get_var("lon");
  c1=LON;
  float lons[LON];
  recd=(*lon_in).get(lons,c1);
  recd=(*lon_out).put(lons,c1);



  time_in=file.get_var("time");
  c1=TIME;
  double times[TIME];
  recd=(*time_in).get(times,c1);
  recd=(*time_out).put(times,c1);

  output.close();
  
  file.close();
  file2.close();
  file3.close();

  //  cout << "Min is: " << min << " Max is: " << max <<endl;

  exit(0);
}


